#include <plib.h>
#include <chipKITEthernet.h>
#include <string.h>
#define USE_SD
#ifdef USE_SD
#include <SD.h>

const int chipSelect = 6;
#endif

//*
#define s8 signed char
#define s16 signed int
#define s32 signed long
#define us8 unsigned char
#define us16 unsigned int
#define us32 long unsigned
//*/
/*
  This sketch is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This sketch is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this sketch; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
/************************************************************************/
/*  Revision History:							*/
/* 									*/
/* 8/08/2011 Digilent Inc. (KeithV)					*/
/* Updated for use with the chipKIT Max32 and chipKIT Network Shield	*/
/*									*/
/************************************************************************/

/*
  Web  Server
 
 A simple web server that shows the value of the analog input pins.
 using an Arduino Wiznet Ethernet shield. 
 
 Circuit:
 * Ethernet shield attached to pins 10, 11, 12, 13
 * Analog inputs attached to pins A0 through A5 (optional)
 
 created 18 Dec 2009
 by David A. Mellis
 modified 4 Sep 2010
 by Tom Igoe
 
 */
void render_web_page_not_found(Client *client);
void render_web_page_simple(Client *client, String *message);
void render_web_page_from_file(Client *client, char *filename);

// Enter a MAC address and IP address for your controller below. 
// A zero MAC address means that the chipKIT MAC is to be used
byte mac[] = {  
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };

//       !!MODIFY THIS!!
// The IP address will be dependent on your local network:
byte ip[] = { 
  192,168,15,190 };

// Initialize the Ethernet server library
// with the IP address and port you want to use 
// (port 80 is default for HTTP):
Server server(80);

long page_count = 0;
char in[24]    = {68, 58, 62, 55, 57,56,63,54,86,64,5,70,22,76,9,2,23,39,8,21,78,79,10,20};
//char in[24]    = {25, 13, 73, 20, 6, 8, 10, 77, 79, 81, 36, 83, 35, 33, 32, 30, 52, 70, 59, 57, 55, 85, 62, 64};
//char out[24]   = {71, 63, 44, 54, 56, 58, 2, 29, 86, 31, 28, 34, 84, 82, 37, 80, 28, 39, 76, 9, 5, 21, 72, 12};

//for mpg
volatile s32 mpg_position = 0; //1073741800;
volatile s32 mpg_position_last = 0;
volatile us8 mpg_changed = 0;

//for pwm
volatile int mymilli = 0;
us8 pwmp[] = {22,76,9,68};
s16 pwms[] = {200,175,5,0};

void setup()
{
//  pinMode(86, INPUT);
//  pinMode(64, INPUT);
//  pinMode(5, INPUT);
//  pinMode(70, INPUT);
//  pinMode(84, INPUT);
//  pinMode(30, INPUT);
 
  char pin;
  for(pin = 0; pin < 24; pin++) {
    pinMode(in[pin], INPUT);
    digitalWrite(in[pin], LOW);
  }
  Serial.begin(115200);
  Serial3.begin(9600);
  delay(3000);

#ifdef USE_SD
  // Setup for SD Card
  Serial.print("Initializing SD card...");
  // make sure that the default chip select pin is set to
  // output, even if you don't use it:
  pinMode(6, OUTPUT);
  
  // see if the card is present and can be initialized:
  if (!SD.begin(chipSelect)) {
    Serial.println("Card failed, or not present");
    // don't do anything more:
    return;
  }
  Serial.println("card initialized.");
#endif

  Serial.print("quicK-240 Web Server Up");
  Serial.println();
  Serial.println();
  // start the Ethernet connection and the server:
  Ethernet.begin(mac, ip);
  //Ethernet.begin();  // this will use DHCP
  server.begin();
  
  //mpg init
  us16 value;
  TRISBSET = 0x0c;
  AD1PCFGSET = 0x0000803c;     //turn analog 2-5,15 off 0x0000403c
  AD1CON2CLR = 0x0000d000; //turn off external voltage compare pins?
  CNCON = 0x00008000;   //turn "interrupt on change" on
  CNEN = 0x00000020;   //enable cn2-7,12,15 //0x000090fc
  CNPUE= 0x00000000; //weak pull up off
  value = PORTB;      //Read the Ports
  IPC6SET = 0x001f0000; //set priority to 7 sub 4
  IFS1CLR = 0x0001; //clear the interupt flag bit
  IEC1SET= 0x0001; // Enable Change Notice interrupts

  //pwm init
  ConfigIntTimer3(T3_INT_ON | T3_INT_PRIOR_3);
  OpenTimer3(T3_ON | T3_PS_1_4, 1000);
  for(int i = 0; i < 4; i++) {
    pinMode(pwmp[i], OUTPUT);
    digitalWrite(pwmp[i], LOW);
  }
}

const int BUF_SIZE = 200;
char buffer[BUF_SIZE];
int bufptr = 0;
enum device {
  MIN,
  motor,
  buzzer,
  led,
  red,
  green,
  blue,
  fan,
  MAX
} selected_device = MIN;

void loop()
{
  String stp101_command;
  if (mpg_changed==1)
  {
    mpg_changed=0;
    if( selected_device == buzzer ) ;

    if( selected_device == led ) {
      if( mpg_position < 0 ) { mpg_position = 1599; }
      else if( mpg_position < 200 )  { pwms[0] = mpg_position % 200;         pwms[1] = 0;                           pwms[2] = 0; }
      else if( mpg_position < 400 )  { pwms[0] = 200;                        pwms[1] = mpg_position % 200;          pwms[2] = 0; }
      else if( mpg_position < 600 )  { pwms[0] = 200;                        pwms[1] = 200;                         pwms[2] = mpg_position % 200; }
      else if( mpg_position < 800 )  { pwms[0] = 200;                        pwms[1] = 200 - (mpg_position % 200);  pwms[2] = 200; }
      else if( mpg_position < 1000 ) { pwms[0] = 200 - (mpg_position % 200); pwms[1] = 0;                           pwms[2] = 200; }
      else if( mpg_position < 1200 ) { pwms[0] = 0;                          pwms[1] = (mpg_position % 200);        pwms[2] = 200; }
      else if( mpg_position < 1400 ) { pwms[0] = 0;                          pwms[1] = 200;                         pwms[2] = 200 - (mpg_position % 200); }
      else if( mpg_position < 1600 ) { pwms[0] = 0;                          pwms[1] = 200 - (mpg_position % 200);  pwms[2] = 0; }
      else { mpg_position = 0; }
    }
    if( selected_device >= red && selected_device <= fan ) {
      pwms[selected_device - red] = (s16) (mpg_position);
      Serial.print(pwms[selected_device - red], DEC);
      Serial.print(" ");
      if( pwms[selected_device - red] > 200 ) {
        pwms[selected_device - red] = 200;
        mpg_position = 200;
      }
      if( pwms[selected_device - red] <   0 ) {
        pwms[selected_device - red] =   0;
        mpg_position = 0;
      }
      Serial.println(pwms[selected_device - red], DEC);
    }

    if( selected_device == motor ) {
      stp101_command = String("MI") + String(mpg_position) + String("\r");
      //stp101_command = String("II") + String(mpg_position - mpg_position_last) + String("\r");
      //Serial.println(stp101_command);
      Serial3.print(stp101_command);
    }
    mpg_position_last = mpg_position;
  }
  
  char command[100];
  char filename[50];
  char form[50];
  
  strcpy(form, "null");
  strcpy(filename, "null");
  
  // listen for incoming clients
  Client client = server.available();
  if (client) {
    // an http request ends with a blank line
    boolean currentLineIsBlank = true;
    boolean render = true;
    while (client.connected()) {
      if (client.available()) {
        char c = client.read();
        // if you've gotten to the end of the line (received a newline
        // character) and the line is blank, the http request has ended,
        // so you can send a reply
      
        buffer[bufptr] = c;
        bufptr++;
        //Serial.print("=");
        //Serial.print(c);
        if (c < 0x20)
        {
          //Serial.print(c,HEX);
        }

        if (c == '\n') {
          buffer[bufptr] = 0;

          //if( !strcmp(buffer,"") ) {
          //  Serial.print(buffer[0],HEX);
          //  Serial.println(" blank line found");
          //}
          if( !strncmp(buffer,"GET",3) ) {
            strcpy(command,buffer);
            Serial.print("*** ");
            Serial.println(buffer);
          }
          
          if( !strncmp(buffer,"GET /favicon.ico",16) ) {
            Serial.println("*GET /favicon.ico*");
            render=false;
          }
          else if( !strncmp(buffer,"GET / ",6) ) {
            Serial.println("*GET / *");
            render=true;
            strcpy(filename, "index.htm");
          }
          else if( !strncmp(buffer,"GET",3) ) {
            Serial.println("*GET*");
            char pin;
            int toknum = 0;
            int i = 0; // skip "GET "
            int j = 0;
            int k = 0;
            us8 parse_state = 0;
            for( ; i < strlen(buffer); i++ ) {
              //Serial.print(buffer[i]);
              //Serial.print(' ');
              //Serial.println(parse_state, HEX);

//#ifdef GONE
              switch( parse_state ) {
                case 0: // look for GET
                  switch( buffer[i] ) {
                    case ' ':
                      parse_state = 1;
                      break;
                    default:
                      form[j++] = buffer[i];
                      form[j] = 0;
                      break;
                  }
                  break;
                case 1: // look /
                  switch( buffer[i] ) {
                    case '/':
                      parse_state = 2;
                      break;
                  }
                  break;
                case 2: // look for file name
                  switch( buffer[i] ) {
                    case '?':
                      parse_state = 3;
                      render=false;
                      Serial.print("file: \"");
                      Serial.print(filename);
                      Serial.println("\"");
                      break;
                    case ' ':
                      parse_state = 4;
                      Serial.print("file: \"");
                      Serial.print(filename);
                      Serial.println("\"");
                      break;
                    default:
                      filename[k++] = buffer[i];
                      filename[k] = 0;
                      break;
                  }
                  break;
                case 3: // look for get message
                  switch( buffer[i] ) {
                    case ' ':
                      parse_state = 4;
                      Serial.print("form: \"");
                      Serial.print(form);
                      Serial.println("\"");
                      if( !strcmp(form, "select=buzzer") ) selected_device = buzzer;
                      if( !strcmp(form, "select=led"   ) ) { selected_device = led; mpg_position = 600; mpg_changed=1; }
                      else { pwms[0] = 0; pwms[1] = 0; pwms[2] = 0; }
                      if( !strcmp(form, "select=red"   ) ) { selected_device = red; mpg_position = pwms[0]; }
                      if( !strcmp(form, "select=green" ) ) { selected_device = green; mpg_position = pwms[1]; }
                      if( !strcmp(form, "select=blue"  ) ) { selected_device = blue; mpg_position = pwms[2]; }
                      if( !strcmp(form, "select=fan"   ) ) { selected_device = fan; mpg_position = pwms[3]; }
                      
                      if( !strcmp(form, "select=motor" ) ) {
                        selected_device = motor;
                        stp101_command = String("SF HM0\r");
                        Serial.println(stp101_command);
                        Serial3.print(stp101_command);
                        mpg_position = 0;
                      }
                      break;
                    default:
                      form[k++] = buffer[i];
                      form[k] = 0;
                      break;
                  }
                  break;
                case 4: // look for HTTP version
                  break;
              }
//#endif
#ifdef GONE
              switch( buffer[i] ) {
                case '?':
                case '&':
                case ' ':
                case '\n':
                  render=false;

                  form[j] = 0;
                  //Serial.print(toknum++);
                  //Serial.print(":");
                  //Serial.println(form);
                                    
                  if( !strncmp(form,"pin",3) ) {
                    char pin;
                    char level;
                    
                    if( form[4] == '=' ) {
                      // 1 digit pin number
                      pin = form[3] - '0';
                      level = form[5];
                    }
                    else if( form[5] == '=' ) {
                      // 2 digit pin number
                      pin = (form[3] - '0') * 10;
                      pin += form[4] - '0';
                      level = form[6];
                    }
                    else {
                      Serial.print("ERROR (parse): ");
                      Serial.println(form);
                    }
                    pin = pin - 1; // page renders with 1-24 not 0-23

                    //Serial.print("pin: ");
                    //Serial.println(pin, DEC);

                    //Serial.print("level: ");
                    //Serial.println(level);
                    if( level == '1' ) 
                    {
                    
                      pinMode(in[pin], OUTPUT);
                      digitalWrite(in[pin], HIGH);
                      //client.print("toggle(pin1); ");
                      //Serial3.print("II400\r");
                    }
                    else 
                    {
                      pinMode(in[pin], OUTPUT);
                      digitalWrite(in[pin], LOW);
                      //Serial3.print("II-400\r");
                    }
                    if( pin == 20 ) {
                      pwms[0] = 0;
                      pwms[1] = 50;
                      pwms[2] = 0;
                      pwms[3] = 50;
                    }
                    if( pin == 21 ) {
                      pwms[0] = 100;
                      pwms[1] = 0;
                      pwms[2] = 25;
                      pwms[3] = 25;
                    }
                    if( pin == 22 ) {
                      pwms[0] = 100;
                      pwms[1] = 50;
                      pwms[2] = 0;
                      pwms[3] = 0;
                      mpg_position = 0;
                    }
                    if( pin == 23 ) {
                      pwms[0] = 25;
                      pwms[1] = 50;
                      pwms[2] = 75;
                      pwms[3] = 100;
                      stp101_command = String("SF HM0\r");
                      Serial.println(stp101_command);
                      Serial3.print(stp101_command);
                    }
                  }
                  j = 0;
                  break;
                default:
                  form[j] = buffer[i];
                  j++;
              }
#endif
            }
          }
          //Serial.print(buffer);
          // GET / HTTP/1.1
          bufptr = 0;
        }
        if (c == '\n' && currentLineIsBlank) {
          ///Serial.print("form: \"");
          //Serial.print(form);
          /*
          for( int i = 0; i < 10; i++ ) {
            Serial.print("filename[");
            Serial.print(i, DEC);
            Serial.print("] = ");
            Serial.println(filename[i],HEX);
            if( filename[i] == 0 ) break;
          }
          */
          //Serial.print("strlen(filename) = ");
          //Serial.println(strlen(filename),HEX);

          //Serial.print("\" file: \"");
          //Serial.print(filename);
          //Serial.println("\"");
        }
        if (c == '\n' && currentLineIsBlank && render == false) {
          String message = String(mpg_position,DEC);
          render_web_page_simple(&client, &message);
          break;
        }
        else if (c == '\n' && currentLineIsBlank && render == true) {
          render = false;
          render_web_page_from_file(&client, filename);
          break;
        }
        if (c == '\n') {
          // you're starting a new line
          currentLineIsBlank = true;
        } 
        else if (c != '\r') {
          // you've gotten a character on the current line
          currentLineIsBlank = false;
        }
      }
    }
    
    // give the web browser time to receive the data
    delay(100);
    // close the connection:
    client.stop();
  }
}

extern "C"
{
  void __ISR(_TIMER_3_VECTOR,ipl3) pwmOn(void)
  {
    mT3ClearIntFlag();  // Clear interrupt flag
    if (mymilli>=199)
    {
      mymilli=0;
      //bool temp = digitalRead(37);
      //digitalWrite(37, temp ^ 1);
    }
    else
      mymilli++;
    for(int i = 0; i < 4; i++) {
      if (mymilli < pwms[i])
        digitalWrite(pwmp[i],HIGH);
      else
        digitalWrite(pwmp[i],LOW);
    }
  }

  void __ISR(_CHANGE_NOTICE_VECTOR, ipl7) CN_Interrupt_ISR(void)
  {
    //bool temp = digitalRead(37);
    //digitalWrite(37, temp ^ 1);

    us16 static lastb=0x0c;
    //us16 static lastd;
    if ((PORTB & 0x08) != (lastb & 0x08)) //cn6
    {
      mpg_changed=1;
      if (PORTB & 0x08) 
      {
        if (PORTB & 0x04)
          mpg_position++;
        else
          mpg_position--;
      }
      else
      {
        if (PORTB & 0x04)
          mpg_position--;
        else
          mpg_position++;
      }
    }
    lastb = PORTB; // Read PORTB to clear mismatch condition
    //lastd = PORTD; // Read PORTD to clear mismatch condition
    IFS1CLR = 0x0001; // Be sure to clear the CN interrupt status
    // flag before exiting the service routine.
  }

} // end extern "C"


/*
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This sketch is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this sketch; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
  Web Server Library Routines
 
 These routines are meant to work with the (original) chipKIT Webserver to serve
 files off of SD and/or render some simple pages or 404 page missing files.
 
 created 2012
 by pontech.com (Jacob Christ, Brian Dobrovodsky and Michael Skoczen)
 
 */

void render_web_page_not_found(Client *client) {
  Serial.println("render_web_page_not_found");
  client->println("HTTP/1.1 404 NOT FOUND");
  client->println("Content-Type: text/plain");
  client->println();
  client->println("PONTECH.COM quicK Web Server");
  client->println();
  client->println("ERROR 404: No matching content found");
  client->println();
}

void render_web_page_simple(Client *client, String *message) {
  Serial.println("render_web_page_simple");
  client->println("HTTP/1.1 200 OK");
  client->println("Content-Type: text/plain");
  client->println();
  client->println(*message);
}

void render_web_page_from_file(Client *client, char *filename) {
  Serial.println("render_web_page_from_file");
  // open the file. note that only one file can be open at a time,
  // so you have to close this one before opening another.
#ifdef USE_SD
  char c[100];
  us8 index = 0;
  File dataFile = SD.open(filename);


  // if the file is available, write to it:
  if (dataFile) {
    // send a standard http response header
    client->println("HTTP/1.1 200 OK");
    client->println("Content-Type: text/html");
    //client->println("Content-Type: text/plain");
    client->println();
    while (dataFile.available()) {
      c[index++] = dataFile.read();
      c[index] = 0;
      if( index >= 99 ) {
        client->print(c);
        index = 0;
        c[index] = 0;
      }
      //Serial.print(c[0]);
    }
    client->print(c);
    dataFile.close();
    client->println("");
  }  
  // if the file isn't open, pop up an error:
  else {
#endif
    render_web_page_not_found(client);
#ifdef USE_SD
  } 
#endif
}


